@@ -1,7 +1,7 @@
 /* ignuit - Educational software for the GNOME, following the Leitner
  * flash-card system.
  *
- * Copyright (C) 2008, 2009 Timothy Richard Musson
+ * Copyright (C) 2008, 2009, 2015 Timothy Richard Musson
  *
  * Email: <trmusson@gmail.com>
  * WWW:   http://homepages.ihug.co.nz/~trmusson/programs.html#ignuit
@@ -64,8 +64,9 @@ struct _Card {
 
 struct _Category {
     gchar       *title;
-    GList       *cards;     /* List of all cards in this category. */
-    CardStyle   card_style; /* Default card appearance. */
+    GList       *cards;      /* List of all cards in this category. */
+    CardStyle   card_style;  /* Default card appearance. */
+    gboolean    fixed_order; /* Should these cards be kept in order? */
 };
 
 
@@ -727,6 +728,20 @@ category_get_card_style (Category *cat)
 }
 
 
+void
+category_set_fixed_order (Category *cat, gboolean fixed)
+{
+    cat->fixed_order = fixed;
+}
+
+
+gboolean
+category_is_fixed_order (Category *cat)
+{
+    return cat->fixed_order;
+}
+
+
 gboolean
 card_tested_today (Card *c)
 {
