@@ -1,6 +1,6 @@
 /* card.h
  *
- * Copyright (C) 2008, 2009 Timothy Richard Musson
+ * Copyright (C) 2008, 2009, 2015 Timothy Richard Musson
  *
  * Email: <trmusson@gmail.com>
  * WWW:   http://homepages.ihug.co.nz/~trmusson/programs.html#ignuit
@@ -154,6 +154,9 @@ guint            category_get_n_untested (Category *cat);
 void             category_set_card_style (Category *cat, gint style);
 gint             category_get_card_style (Category *cat);
 
+void             category_set_fixed_order (Category *cat, gboolean fixed);
+gboolean         category_is_fixed_order (Category *cat);
+
 gboolean         card_tested_today (Card *c);
 
 Card*            card_get_details (Card *c);
