@@ -37,6 +37,11 @@
 #define HISTORY_LENGTH   10
 
 
+typedef const gchar* (*GetCardTextFunc)(Card *card);
+typedef gchar* (*StrStrFunc)(const char *haystack, const char *needle);
+typedef gint (*StrCmpFunc)(const char *s1, const char *s2);
+
+
 typedef struct {
 
     Ignuit              *ig;
@@ -47,13 +52,12 @@ typedef struct {
     GtkWidget           *t_case;
     GtkWidget           *t_markup;
 
+    StrStrFunc          my_strstr;
+    StrCmpFunc          my_strcmp;
+
 } Dialog;
 
 
-typedef const gchar* (*GetCardTextFunc)(Card *card);
-typedef gchar* (*StrStrFunc)(const char *haystack, const char *needle);
-
-
 static Dialog *dialog = NULL;
 
 
@@ -61,6 +65,7 @@ enum {
     RADIO_FRONTS = 0,
     RADIO_BACKS,
     RADIO_BOTH,
+    RADIO_TAGS,
     N_RADIO_BUTTONS
 };
 
@@ -68,20 +73,37 @@ enum {
 static gchar*
 utf8_strcasestr (const gchar *haystack, const gchar *needle)
 {
-    gchar *s1, *s2, *ret;
+    gchar *haystack_fold, *needle_fold, *ret;
 
     /* Return non-NULL if haystack contains the substring needle, case
-     * insensitive. Note that the returned value must not be used for
-     * anything other than checking whether or not it's NULL.
-     */
+     * insensitive. Note that the returned value *must not* be used for
+     * anything other than checking whether or not it's NULL. */
 
-    s1 = g_utf8_casefold (haystack, -1);
-    s2 = g_utf8_casefold (needle, -1);
+    haystack_fold = g_utf8_casefold (haystack, -1);
+    needle_fold = g_utf8_casefold (needle, -1);
 
-    ret = strstr (s1, s2);
+    ret = strstr (haystack_fold, needle_fold);
 
-    g_free (s1);
-    g_free (s2);
+    g_free (haystack_fold);
+    g_free (needle_fold);
+
+    return ret;
+}
+
+
+static gint
+utf8_strcasecmp (const gchar *s1, const gchar *s2)
+{
+    gchar *s1_fold, *s2_fold;
+    gint ret;
+
+    s1_fold = g_utf8_casefold (s1, -1);
+    s2_fold = g_utf8_casefold (s2, -1);
+
+    ret = g_utf8_collate (s1_fold, s2_fold);
+
+    g_free (s1_fold);
+    g_free (s2_fold);
 
     return ret;
 }
@@ -96,34 +118,162 @@ get_radio_selection (GSList *group, gint n)
 
         n--;
 
-        if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(cur->data))) {
+        if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(cur->data)))
             return n;
-        }
+
     }
     g_assert_not_reached ();
 }
 
 
+void
+cb_r_tags_toggled (GtkToggleButton *widget, Dialog *d)
+{
+    gboolean active;
+
+    active = gtk_toggle_button_get_active (widget);
+    gtk_widget_set_sensitive (d->t_markup, !active);
+}
+
+
+static void
+search_tags (Dialog *d, GRegex *re, const gchar *text)
+{
+    GList *p, *q, *tlist_card, *tlist = NULL;
+    gboolean changed = FALSE;
+
+
+    if (text[0] == '\0') {
+
+        /* Find cards lacking tags. */
+
+        for (p = file_get_cards (d->ig->file); p != NULL; p = p->next) {
+            if (card_get_tags (CARD(p)) == NULL) {
+                file_add_search_card (d->ig->file, CARD(p));
+            }
+        }
+        return;
+    }
+
+
+    /* Get a list of tags that match the query. */
+
+    for (p = file_get_tags (d->ig->file); p != NULL; p = p->next) {
+        if (re != NULL) {
+            if (g_regex_match (re, tag_get_name (TAG(p)), 0, NULL)) {
+                tlist = tag_list_add_tag (tlist, TAG(p), &changed);
+            }
+        }
+        else {
+            if (d->my_strcmp (tag_get_name (TAG(p)), text) == 0) {
+                tlist = tag_list_add_tag (tlist, TAG(p), &changed);
+                /* This'll be the only one, so no need to continue. */
+                break;
+            }
+        }
+    }
+
+
+    /* Find all cards marked with those tags. */
+
+    for (p = file_get_cards (d->ig->file); p != NULL; p = p->next) {
+
+        tlist_card = card_get_tags (CARD(p));
+
+        for (q = tlist; q != NULL; q = q->next) {
+            if (tag_list_lookup_tag (tlist_card, TAG(q)) != NULL) {
+                file_add_search_card (d->ig->file, CARD(p));
+                /* Done with this card, so: */
+                break;
+            }
+        }
+
+    }
+
+    tag_list_free (tlist);
+}
+
+
+static void
+search_cards (Dialog *d, gboolean fronts, gboolean backs, GRegex *re,
+    const gchar *text)
+{
+    GetCardTextFunc get_front;
+    GetCardTextFunc get_back;
+    GList *p;
+    gboolean hit;
+    Category *cat;
+    Card *c;
+
+
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(d->t_markup))) {
+        get_front = card_get_front;
+        get_back = card_get_back;
+    }
+    else {
+        get_front = card_get_front_without_markup;
+        get_back = card_get_back_without_markup;
+    }
+
+    for (p = file_get_cards (d->ig->file); p != NULL; p = p->next) {
+
+        hit = FALSE;
+
+        c = CARD(p);
+
+        cat = card_get_category (c);
+
+        if (fronts) {
+
+            if (text[0] == '\0') {
+                /* Find cards with no front text. */
+                hit = card_front_is_blank (c);
+            }
+            else if (re != NULL) {
+                hit = g_regex_match (re, get_front (c), 0, NULL);
+            }
+            else {
+                hit = (d->my_strstr (get_front (c), text) != NULL);
+            }
+        }
+
+        if (backs) {
+
+            if (text[0] == '\0') {
+                /* Find cards with no back text. */
+                hit |= card_back_is_blank (c);
+            }
+            else if (re != NULL) {
+                hit |= g_regex_match (re, get_back (c), 0, NULL);
+            }
+            else {
+                hit |= (d->my_strstr (get_back (c), text) != NULL);
+            }
+        }
+
+        if (hit)
+            file_add_search_card (d->ig->file, c);
+
+    }
+}
+
+
 static void
 cb_find (GtkWidget *widget, Dialog *d)
 {
-    GetCardTextFunc get_front, get_back;
-    StrStrFunc my_strstr;
     GRegex *re = NULL;
     GtkComboBox *combo;
-    gboolean fronts, backs, hit, use_regex, match_case;
+    gboolean match_case;
     gchar *text;
-    GList *cur, *cards;
+    GList *cards;
     Category *cat;
-    Card *c;
+    gint i;
 
 
     combo = GTK_COMBO_BOX(d->combo_entry);
 
-    text = gtk_combo_box_get_active_text (combo);
-    if (text == NULL) {
+    if ((text = gtk_combo_box_get_active_text (combo)) == NULL)
         return;
-    }
 
     dialog_editor_check_changed ();
 
@@ -149,103 +299,52 @@ cb_find (GtkWidget *widget, Dialog *d)
         }
     }
 
-    fronts = TRUE;
-    backs = TRUE;
-
-    switch (get_radio_selection (d->face_group, N_RADIO_BUTTONS)) {
-    case RADIO_FRONTS:
-        backs = FALSE;
-        break;
-    case RADIO_BACKS:
-        fronts = FALSE;
-        break;
-    case RADIO_BOTH:
-        break;
-    }
 
     /* Clear any previous search results. */
     file_clear_search (d->ig->file, TRUE);
 
 
     match_case = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(d->t_case));
-    my_strstr = match_case ? strstr : utf8_strcasestr;
 
-    use_regex = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(d->t_regex));
-    if (use_regex) {
+    d->my_strstr = match_case ? strstr : utf8_strcasestr;
+    d->my_strcmp = match_case ? strcmp : utf8_strcasecmp;
+
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(d->t_regex)) == TRUE) {
 
         GError *err = NULL;
 
         re = g_regex_new (text, match_case ? 0 : G_REGEX_CASELESS, 0, &err);
         if (err != NULL) {
-
-            /* XXX: should we bother with a pop-up message here? */
-            /* Or maybe set the entry widget's colour to red? */
-
             g_warning ("%s\n", err->message);
-
-            use_regex = FALSE;
             g_error_free (err);
+            re = NULL; /* Fall back to plain search. */
         }
     }
 
 
-    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(d->t_markup))) {
-        get_front = card_get_front;
-        get_back = card_get_back;
+    i = get_radio_selection (d->face_group, N_RADIO_BUTTONS);
+
+    if (i == RADIO_TAGS) {
+        search_tags (d, re, text);
     }
     else {
-        get_front = card_get_front_without_markup;
-        get_back = card_get_back_without_markup;
+
+        gboolean fronts;
+        gboolean backs;
+
+        fronts = (i == RADIO_FRONTS || i == RADIO_BOTH);
+        backs = (i == RADIO_BACKS || i == RADIO_BOTH);
+
+        search_cards (d, fronts, backs, re, text);
     }
 
 
-    for (cur = file_get_cards (d->ig->file); cur != NULL; cur = cur->next) {
-
-        hit = FALSE;
-
-        c = CARD(cur);
-
-        cat = card_get_category (c);
-        g_assert (cat != NULL);
-
-        if (fronts) {
-            if (text[0] == '\0') {
-                hit = card_front_is_blank (c);
-            }
-            else if (use_regex) {
-                hit = g_regex_match (re, get_front (c), 0, NULL);
-            }
-            else {
-                hit = (my_strstr (get_front (c), text) != NULL);
-            }
-        }
-
-        if (backs) {
-            if (text[0] == '\0') {
-                hit |= card_back_is_blank (c);
-            }
-            else if (use_regex) {
-                hit |= g_regex_match (re, get_back (c), 0, NULL);
-            }
-            else {
-                hit |= (my_strstr (get_back (c), text) != NULL);
-            }
-        }
-
-        if (hit) {
-            file_add_search_card (d->ig->file, c);
-        }
-
-    }
-
-    if (re != NULL) {
+    if (re != NULL)
         g_regex_unref (re);
-    }
 
     /* If text wasn't added to ig->recent_search_terms, free it now. */
-    if (text[0] == '\0') {
+    if (text[0] == '\0')
         g_free (text);
-    }
 
     cat = file_get_search (d->ig->file);
     cards = category_get_cards (cat);
@@ -264,17 +363,21 @@ cb_find (GtkWidget *widget, Dialog *d)
     gtk_widget_set_sensitive (d->ig->b_remove_category, FALSE);
     gtk_widget_set_sensitive (d->ig->m_add_card, FALSE);
     gtk_widget_set_sensitive (d->ig->b_add_card, FALSE);
+    gtk_widget_set_sensitive (d->ig->m_edit_tags, cards != NULL);
+    gtk_widget_set_sensitive (d->ig->m_flag, cards != NULL);
     gtk_widget_set_sensitive (d->ig->m_switch_sides, cards != NULL);
     gtk_widget_set_sensitive (d->ig->m_reset_stats, cards != NULL);
     gtk_widget_set_sensitive (d->ig->m_paste_card, FALSE);
 
+    gtk_widget_set_sensitive (d->ig->m_card_popup_edit_tags, cards != NULL);
+    gtk_widget_set_sensitive (d->ig->m_card_popup_flag, cards != NULL);
     gtk_widget_set_sensitive (d->ig->m_card_popup_switch_sides, cards != NULL);
     gtk_widget_set_sensitive (d->ig->m_card_popup_reset_stats, cards != NULL);
     gtk_widget_set_sensitive (d->ig->m_card_popup_paste, FALSE);
 
     app_window_update_appbar (d->ig);
 
-    dialog_editor_tweak ();
+    dialog_editor_tweak (ED_TWEAK_ALL);
 }
 
 
@@ -300,7 +403,7 @@ void
 dialog_find (Ignuit *ig)
 {
     Dialog    *d;
-    GtkWidget *w, *b_close, *b_find, *hbox, *label, *entry;
+    GtkWidget *w, *b_close, *b_find, *hbox, *label, *entry, *r_tags;
     gchar     *glade_file;
     GladeXML  *glade_xml;
     GList     *cur;
@@ -328,7 +431,6 @@ dialog_find (Ignuit *ig)
     d->window = glade_xml_get_widget (glade_xml, "dialog");
 
     label = glade_xml_get_widget (glade_xml, "label_search");
-    d->combo_entry = glade_xml_get_widget (glade_xml, "combo_entry");
 
     b_close = glade_xml_get_widget (glade_xml, "b_close");
     b_find = glade_xml_get_widget (glade_xml, "b_find");
@@ -340,10 +442,9 @@ dialog_find (Ignuit *ig)
 
     gtk_label_set_mnemonic_widget (GTK_LABEL(label), d->combo_entry);
 
-    for (cur = ig->recent_search_terms; cur != NULL; cur = cur->next) {
+    for (cur = ig->recent_search_terms; cur != NULL; cur = cur->next)
         gtk_combo_box_append_text (GTK_COMBO_BOX(d->combo_entry),
             (gchar*)cur->data);
-    }
 
     entry = gtk_bin_get_child (GTK_BIN(d->combo_entry));
 
@@ -353,6 +454,8 @@ dialog_find (Ignuit *ig)
     w = glade_xml_get_widget (glade_xml, "r_both");
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), TRUE);
 
+    r_tags = glade_xml_get_widget (glade_xml, "r_tags");
+
     d->t_regex = glade_xml_get_widget (glade_xml, "toggle_regex");
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(d->t_regex),
         prefs_get_find_with_regex (d->ig->prefs));
@@ -365,6 +468,8 @@ dialog_find (Ignuit *ig)
 
     g_signal_connect (G_OBJECT(entry), "activate",
         G_CALLBACK(cb_find), d);
+    g_signal_connect (G_OBJECT(r_tags), "toggled",
+        G_CALLBACK(cb_r_tags_toggled), d);
     g_signal_connect (G_OBJECT(b_close), "clicked",
         G_CALLBACK(cb_close), d);
     g_signal_connect (G_OBJECT(b_find), "clicked",
