@@ -104,12 +104,12 @@ latex_init (void)
 }
 
 
-void
+static void
 latex_set_dpi (gint n)
 {
-    if (n < 1 || n > 500) {
+    if (n < 1 || n > MAX_LATEX_DPI)
         return;
-    }
+
     dpi = n;
 }
 
@@ -120,12 +120,11 @@ latex_free (void)
     g_string_free (gstr, TRUE);
     texfiles_free ();
 
-    if (re_math_inline != NULL) {
+    if (re_math_inline != NULL)
         g_regex_unref (re_math_inline);
-    }
-    if (re_latex != NULL) {
+
+    if (re_latex != NULL)
         g_regex_unref (re_latex);
-    }
 }
 
 
@@ -159,12 +158,11 @@ latex_render (gboolean math_inline)
             g_free (s);
         }
 
-        if (preamble == NULL) {
+        if (preamble == NULL)
             preamble = g_strdup (PREAMBLE);
-        }
-        if (postamble == NULL) {
+
+        if (postamble == NULL)
             postamble = g_strdup (POSTAMBLE);
-        }
 
         s = g_strdup_printf ("%s%s%s", preamble, gstr->str, postamble);
 #if 0
@@ -287,12 +285,14 @@ re_latex_eval (GMatchInfo *match_info, GString *result, gpointer data)
 
 
 gchar*
-latex_preprocess (const gchar *text, const gchar *wdir, gboolean do_render)
+latex_preprocess (const gchar *text, const gchar *wdir, gint latex_dpi,
+    gboolean do_render)
 {
     gchar *s1, *s2;
 
     texfiles_free ();
 
+    latex_set_dpi (latex_dpi);
     render = do_render;
     workdir = wdir;
 
